/*
    druckgott get RPM to Mach3 with:
 Orange RPM Sensor: http://www.hobbyking.com/hobbyking/store/uh_viewItem.asp?idProduct=66519
 
 Run Brushless Motor get PWM from Mach3 and change it to RC Sigal
 */


#include <Servo.h> 

Servo myservo;

#define DEBUG NULL  

//CONFIG  
#define inPin  4     // pwm input from Mach3 to this Pin (Sub Pin 11 on Stepcraft)
#define runPin 5  // Connection for starting/stop Signal of the Spindle (Sub Pin 10 on Stepcraft)

#define escPin 6    // Connection for Brushless Motor to Signal pin (orange/yellow)
//#define escMAX 180  // 0 - 180 Grad fuer ein Servro oder Regler

#define POLES 4
#define esc_MOTOR_ON 2000  // Motor on ys
#define esc_MOTOR_OFF 1000  // Motor off ys

#define PWM_PERIOD  100000 // the number of microseconds for one PWM cycle which is configurate on Mach3
#define PWM_IDLE_STATE  HIGH   // set this to HIGH or LOW depending on how the mach3 outputs PWM

#define RPM_TO_MACH3 12 //Pin where Mach3 will read the rpm signal (Sub Pin SUBin on Stepcraft)
#define INTERRUPT_PIN 0   // interrupt Pin: digital pin 2 http://arduino.cc/en/pmwiki.php?n=Reference/AttachInterrupt


int interruptCount = 0;
int trigger = 0; //trigger for low and high output of rpm signal to mach3
long old_pwm;
int check_running = NULL;

void setup()
{
  myservo.attach(escPin); //Initial the Servo
  pinMode(runPin, INPUT); //Initial Start/Stop Pin

  // setup for Output Pin MACH3
  pinMode(RPM_TO_MACH3, OUTPUT);
  // setup interrupt pin
  pinMode(INTERRUPT_PIN, INPUT);
  attachInterrupt(INTERRUPT_PIN, interruptFired, CHANGE);

  if(DEBUG){
    Serial.begin(9600);
  }
}

void loop()
{
  check_running=digitalRead(runPin);
  //check if the spindle should run
  if(check_running == LOW){
    myservo.writeMicroseconds(esc_MOTOR_OFF);
    if(DEBUG){
      Serial.println(" Motor off");
    }
  }

  if(check_running == HIGH){
    //get PWM signal from Mach3
    long  pwmIn = pulseIn( inPin,  PWM_IDLE_STATE );
    //Pruefen ob die variable gesetzt ist, sonst alten Variable nehmen
    if(pwmIn==NULL){
      pwmIn=old_pwm;
    }
    else{
      old_pwm=pwmIn;
    }
    //Map PWM Signal from Mach3 to Servo Signal
    long escOut2 = map(pwmIn, 0, PWM_PERIOD, esc_MOTOR_OFF, esc_MOTOR_ON);
    //Move Servo/esc 
    myservo.writeMicroseconds(escOut2);
    if(DEBUG){
      Serial.print("Mach3 in: ");
      Serial.print(pwmIn);
      Serial.print(" ESC out: ");
      Serial.println(escOut2);
    }
  }

  //delay(500);
}



//Fire Count interrupt
void interruptFired()
{

  //switch between low and high signal output for rpm input to mach3
  switch (trigger) {
  case 1:
    if(interruptCount == POLES*2){
      digitalWrite(RPM_TO_MACH3, HIGH);   //set HIGH
      //trigger 2 for next interrupt to switch to low
      trigger=2;			
      interruptCount=0;
    }
    break;
  case 2:
    if(interruptCount == POLES*2){
      digitalWrite(RPM_TO_MACH3, LOW);    //set LOW
      //trigger 1 for next interrupt to switch to high
      trigger=1;
      interruptCount=0;
    }
    break;
  default:
    digitalWrite(RPM_TO_MACH3, LOW);    
    trigger=1;       
    interruptCount=0;
  }

  interruptCount++;
}


